@@ -30,4 +30,4 @@ class InitError(AEOutputError):
   '''
   def __init__(self, *args, **kwargs):
     AEOutputError.__init__(self,
-      _('The AEOutput device could not be initialized'), *args, **kwargs)
\ No newline at end of file
+      _('The output device could not be initialized'), *args, **kwargs)
\ No newline at end of file
