@@ -4,20 +4,24 @@ monitors.
 
 @author: Larry Weiss
 @author: Peter Parente
+@author: Brett Clippingdale
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005 IBM Corporation
+@copyright: Copyright (c) 2005, 2006 IBM Corporation
 @license: Common Public License 1.0
 
 All rights reserved. This program and the accompanying materials are made
 available under the terms of the Common Public License v1.0 which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/cpl1.0.php}
+
+@todo: DBC: add methods to play sound, ie. play(dev, file)
 '''
 import logging, sys
 import AEOutput, AEInput, AEEvent
 from LSRInterfaces import implements
-from UIRegistrar import UIRegistrar, DEVICE, MONITOR, STARTUP
+from UIRegistrar import UIRegistrar, DEVICE, MONITOR
 from i18n import _
+import LSRConstants
 
 log = logging.getLogger('Device')
 
@@ -25,8 +29,18 @@ class DeviceManager(object):
   '''
   Creates and manages the devices and monitors use by LSR. Keeps a list of each
   and also defines a "default" for devices. Provides a common interface to all
-  output and input devices and mirrors I/O to the monitors.
+  output and input devices and mirrors I/O to the monitors. Provides a mapping
+  from output styles to semantic concepts in the user interface to allow
+  customization of how information is presented.
   
+  @ivar dirty_style: Tracks whether a style has been fetched or not for 
+    modification. If it has, the L{sendString} should also send the style to
+    a device.
+  @type dirty_style: boolean
+  @ivar last_style: The last L{AEOutput.Style} object sent to a device. Used
+    to determine if the next L{sendString} should send a style or not. Not
+    sending a style is an optimization.
+  @type last_style: L{AEOutput.Style}
   @ivar event_manager: L{EventManager} to which this L{DeviceManager} can post 
     L{AEEvent}s 
   @type event_manager: L{EventManager}
@@ -44,6 +58,10 @@ class DeviceManager(object):
   @type def_input: L{AEInput}
   @ivar acc_eng: The AccessEngine reference that created this class
   @type acc_eng: L{AccessEngine}
+  @ivar styles: Dictionary of L{AEOutput.Style}s populated on device 
+    registration in L{_registerOutputDevice}. Keys are a 3-tuple of device,
+    semantic, and layer.
+  @type styles: dictionary of L{AEOutput.Style}
   '''
   def __init__(self, acc_eng):
     '''
@@ -60,10 +78,13 @@ class DeviceManager(object):
     self.def_input = None
     self.acc_eng = acc_eng
     self.event_manager = None
+    self.styles = {}
+    self.dirty_style = True
+    self.last_style = None
 
   def init(self, event_man, **kwargs):
     '''
-    Loads output and input devices. Called by L{AccessEngine} at startup.
+    Called by L{AccessEngine} at startup.
     
     @todo: PP: how do we unload these devices when they're no longer desired?
 
@@ -76,7 +97,7 @@ class DeviceManager(object):
     self.event_manager = event_man
     reg = UIRegistrar()
     # load all device classes to be created at startup
-    devs = reg.loadAssociated(DEVICE, self.acc_eng.getProfile(), STARTUP)
+    devs = reg.loadAssociated(DEVICE, self.acc_eng.getProfile())
     # register all devices, making the first input and output the defaults
     for d in devs:
       # call registerDevice which will automatically set defaults
@@ -87,7 +108,7 @@ class DeviceManager(object):
       except NotImplementedError:
         log.info('Device does not provide an I/O interface: %s', str(d))
     # load all I/O monitor classes to be created at startup
-    mons = reg.loadAssociated(MONITOR, self.acc_eng.getProfile(), STARTUP)
+    mons = reg.loadAssociated(MONITOR, self.acc_eng.getProfile())
     for m in mons:
       # call registerDevice which will weed out non-IO monitors for us
       try:
@@ -122,7 +143,7 @@ class DeviceManager(object):
         # ignore all exceptions
         pass
       
-  def stopNow(self, dev):
+  def sendStop(self, dev, sem, layer):
     '''
     Sends the stop command to the referenced output device. When the device has
     not been registered or is None, the stop command is sent to the default
@@ -131,29 +152,57 @@ class DeviceManager(object):
     @param dev: The device to send the stop command or None to send to L{the 
       default output device<setDefaultOutput>}.
     @type dev: L{AEOutput}
+    @param sem: Semantic description of the information to stop None to indicate
+      stopping all output
+    @type sem: integer
+    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
+      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @type layer: integer
     '''
     # when dev is None assume defaultOutput (which may also be None)
     dev = dev or self.def_output
-    dev.sendStop()
-    # always send the command to the registered monitors
-    self.showStop(dev)
+    if sem is None and layer is None:
+      self.showStop(dev, sem, layer, None)
+      dev.sendStop(None)
+    elif sem is None:
+      # find all styles associated with the given layer
+      all = set([self.styles[(sem, layer, dev)] for sem in
+                 LSRConstants.SEMANTIC_STYLES.keys()])
+      # send stops to all associated
+      for s in all:
+        self.showStop(dev, sem, layer, s)
+        dev.sendStop(s)
+    else:
+      s = self.styles[(sem, layer, dev)]
+      dev.sendStop(s)
+      self.showStop(dev, sem, layer, s)
+    # reset the last style flag so that it is resent on the next sendString,
+    # otherwise, the stop my prempt the application of the last style
+    self.last_style = None
 
-  def showStop(self, dev):
+  def showStop(self, dev, sem, layer, style):
     '''
     Calls all registered output monitors to show that a stop command has been
     sent to the referenced output device. This does nothing when no monitors
     are registered.
 
-    @param dev: The device that was sent the stop command.
+    @param dev: The device that was sent the stop command
     @type dev: L{AEOutput}
+    @param sem: Semantic description of the output that was stopped
+    @type sem: integer
+    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
+      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @type layer: integer
+    @param style: Style associated with the semantic 
+    @type style: L{AEOutput.Style}
     '''
     for mon in self.out_mons:
-      mon.showStop(dev)
+      mon.showStop(dev, sem, layer, style)
     # for demonstration purposes, just print if no monitors
-    # if not self.out_mons:
+    #if not self.out_mons:
     #  sys.stdout.write('{stop}')
 
-  def talkNow(self, dev):
+  def sendTalk(self, dev, sem, layer):
     '''
     Tells the specified output device to send buffered data. When the device
     has not been registered or is None, the talk command is sent to the
@@ -162,14 +211,24 @@ class DeviceManager(object):
     @param dev: The device to send the talk command or None to send to L{the 
       default output device<setDefaultOutput>}
     @type dev: L{AEOutput}
+    @param sem: Semantic information to start outputting or None to indicate 
+      that all buffered information should be output
+    @type sem: integer
+    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
+      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @type layer: integer
     '''
     # when dev is None assume defaultOutput (which may also be None)
     dev = dev or self.def_output
-    dev.sendTalk()
+    if sem is None:
+      s = None
+    else:
+      s = self.styles[(sem, layer, dev)]
+    dev.sendTalk(s)
     # always send the command to the registered monitors
-    self.showTalk(dev)
+    self.showTalk(dev, sem, layer, s)
 
-  def showTalk(self, dev):
+  def showTalk(self, dev, sem, layer, style):
     '''
     Calls all registered output monitors to show the talk command sent to the
     referenced output device. This does nothing when no monitors are
@@ -177,49 +236,61 @@ class DeviceManager(object):
 
     @param dev: The device that was sent the talk command.
     @type dev: L{AEOutput}
+    @param sem: Semantic information that was output
+    @type sem: integer 
+    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
+      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @type layer: integer
+    @param style: Style associated with the semantic 
+    @type style: L{AEOutput.Style}
     '''
     for mon in self.out_mons:
-      mon.showTalk(dev)
+      mon.showTalk(dev, sem, layer, style)
     # for demonstration purposes, just print if no monitors
     #if not self.out_mons:
     #  print '{talk}'
     
-  def sendString(self, dev, string, talk=True):
+  def sendString(self, dev, text, sem, layer, talk=True):
     '''
     Sends the string to the specified output device. When the device has not
     been registered or is None, the string is sent to the default output
     device. The string is sent with a L{AEOutput.Base.AEOutput.sendString} call
-    followed by L{talkNow} when the talk parameter is true.
-    
-    @note: On 12/13/05 PP changed this method to send more than one character at
-    a time to each device. Whatever string is passed to this method is now 
-    sent as-is to each device. The default behavior of any Speech device is
-    to add whatever it receives to its output thread queue. In this case, each
-    string is added as an element to the queue. The thread then dumps whatever
-    is in its queue to the output device driver. In this case, the driver is
-    now receiving entire words to buffer and later send when the buffer is full
-    or a talk is received. This small change results in drastic performance
-    improvements to LSR as the call overhead is now greatly reduced from here
-    to each device, from each device object to its output thread, and from the
-    output thread to the device hardware.
+    followed by L{sendTalk} when the talk parameter is True.
 
     @param dev: The device to send the string or None to send to L{the default 
       output device<setDefaultOutput>}
     @type dev: L{AEOutput}
-    @param string: The string to send to the referenced device
-    @type string: string
-    @param talk: Defaults to True to indicate that L{talkNow} should be called
+    @param text: The text to send to the referenced device
+    @type text: string
+    @param sem: Semantic description of the text to send or None to indicate the
+      information is void of any known semantic
+    @type sem: integer 
+    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
+      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @type layer: integer
+    @param talk: Defaults to True to indicate that L{sendTalk} should be called
       after all characters are sent
     @type talk: boolean
     '''
     # when dev is None assume defaultOutput (which may also be None)
     dev = dev or self.def_output
-    dev.sendString(string)
+    s = self.styles[(sem, layer, dev)]
+    # respect muting
+    if s.Mute: return
+    # if the style hasn't changed, and the last style is the same one as the
+    # one we're trying to send now, don't resend it
+    if not self.dirty_style and self.last_style == s:
+      s = None
+    else:
+      # keep track of the last style sent and reset the dirty flag
+      self.last_style = s
+      self.dirty_style = False
+    dev.sendString(text, s)
     if talk:
-       self.talkNow(dev)
-    self.showString(dev, string, talk)
- 
-  def showString(self, dev, string, talk):
+       self.sendTalk(dev, sem, layer)
+    self.showString(dev, text, sem, layer, s, talk)
+
+  def showString(self, dev, text, sem, layer, style, talk):
     '''
     Sends a string to all registered output monitors. This method should be
     used when only monitor output is desired without sending to any devices.
@@ -229,125 +300,47 @@ class DeviceManager(object):
 
     @param dev: The device that was sent the string.
     @type dev: L{AEOutput}
-    @param string: The string that was sent to the referenced device
-    @type string: string
-    @param talk: Defaults to True to indicate that L{talkNow} should be called
-    after all characters are sent
+    @param text: The text that was sent to the referenced device
+    @type text: string
+    @param sem: Semantic description of the text sent
+    @type sem: integer
+    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
+      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @type layer: integer
+    @param style: Style associated with the semantic 
+    @type style: L{AEOutput.Style}
+    @param talk: Indicates that L{sendTalk} was called
     @type talk: boolean
     '''
     for mon in self.out_mons:
-      mon.showString(dev, string, talk)
+      mon.showString(dev, text, sem, layer, style, talk)
     # for demonstration purposes, just print if no monitors
     #if not self.out_mons:
-      #print string
+      #if talk:
+        #print text
+      #else:
+        #for ch in text:
+          #sys.stdout.write(ch)
 
-  def sendCmd(self, dev, cmd, value):
-    '''
-    Sends the referenced command to the referenced device. When the device has
-    not been registered or is None, the command is sent to the default output
-    device. After sending the command, L{showCmd} is called.
-
-    @param dev: The device to send the command or None to send to L{the default 
-      output device<setDefaultOutput>}
-    @type dev: L{AEOutput}
-    @param cmd: The device specific command that should be sent
-    @type cmd: integer
-    @param value: The device specific value for the specified command
-    @type value: object
-    @raise ValueError: When the command or value is invalid
-    '''
-    # when dev is None assume defaultOutput (which may also be None)
-    dev = dev or self.def_output
-    dev.sendCommand(cmd, value)
-    # always send the command to the registered monitors
-    self.showCmd(dev, cmd, value)
-
-  def showCmd(self, dev, cmd, value):
-    '''
-    Calls all registered output monitors to show the command sent to the
-    referenced output device. This does nothing when no output monitors are
-    registered.
-
-    @param dev: The device that was sent the command
-    @type dev: L{AEOutput}
-    @param cmd: The device specific command that should be sent
-    @type cmd: int
-    @param value: The device specific value for the specified command
-    @type value: float
-    '''
-    for mon in self.out_mons:
-      mon.showCmd(dev, cmd, value)
-    # for demonstration purposes, just print if no monitors
-    #if not self.out_mons:
-    #  sys.stdout.write('{CMD:%s(%d,%d)}' % (dev.getName(), cmd, value))
-
-  def getCmdLevel(self, dev, cmd):
-    '''
-    Gets the current value of the specified command.
-
-    The default value should be returned when it has not been changed by
-    L{sendCmd}. A return of 0 indicates the the specified command has only a
-    single value. A return of -1 indicates that either the device is not
-    registered, or the referenced command is not available.
-
-    @param dev: The device to query or None to query L{the default output 
-      device<setDefaultOutput>}
-    @type dev: L{AEOutput}
-    @param cmd: The device specific command to query
-    @type cmd: int
-    @return: The current value of the specified command
-    @rtype: int
-    '''
-    value = -1
-    # when dev is None assume defaultOutput (which may also be None)
-    if dev is None:
-      dev = self.def_output
-    if dev in self.out_devs:
-      try:
-        value = dev.getCommandLevel(cmd)
-      except ValueError:
-        pass  # value already set to -1
-    # return the value
-    return value
-
-  def getMaxLevel(self, dev, cmd):
-    '''
-    Gets the maximum value of the specified command.
-
-    A return of -1 indicates that either the device is not registered, or the 
-    referenced command is not available.
-
-    @param dev: The device to query or None to query L{the default output 
-      device<setDefaultOutput>}
-    @type dev: L{AEOutput}
-    @param cmd: The device specific command to query
-    @type cmd: int
-    @return: The maximum value of the specified command
-    @rtype: int
-    '''
-    value = -1
-    # when dev is None assume defaultOutput (which may also be None)
-    if dev is None:
-      dev = self.def_output
-    if dev in self.out_devs:
-      try:
-        value = dev.getLevelCount(cmd)
-      except ValueError:
-        pass  # value already set to -1
-    # return the value
-    return value
-
-  def sendIndex(self, dev, marker=0):
+  def sendIndex(self, dev, sem, layer, por):
     '''
     Sends the referenced index marker to the referenced device. When the
     device has not been registered or is None, the index marker is sent to the
-    default output device. After sending the command, L{showCmd} is called.
+    default output device. After sending the command, L{showIndex} is called.
 
     @param dev: The device to send the index marker or None to send to L{the 
       default output device<setDefaultOutput>}
     @type dev: L{AEOutput}
-    @param marker: The marker value that should be sent or 0 when unspecified
-    @type marker: int
+    @param sem: Semantic description of the index text to send or None to 
+      indicate the information is void of any known semantic
+    @type sem: integer
+    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
+      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @type layer: integer
+    @param por: Point of regard associated with the index marker
+    @type por: L{POR}
+
+    @todo: PP: rewrite for POR
     '''
     # when dev is None assume defaultOutput (which may also be None)
     if dev is None:
@@ -356,12 +349,12 @@ class DeviceManager(object):
       if dev in self.out_devs:
         dev.sendIndex(marker)
       # always send the command to the registered monitors
-      self.showIndex(dev, marker)
+      self.showIndex(dev, sem, layer, marker)
     except NotImplementedError:
-      # do nothing when Indexing not implemented
+      # do nothing when indexing not implemented
       pass
 
-  def showIndex(self, dev, marker):
+  def showIndex(self, dev, sem, layer, por):
     '''
     Calls all registered output monitors to show the index marker sent to the
     referenced output device. This does nothing when no output monitors are
@@ -369,11 +362,19 @@ class DeviceManager(object):
 
     @param dev: The device that was sent the index marker
     @type dev: L{AEOutput}
-    @param marker: The marker value that was sent
-    @type marker: int
+    @param sem: Semantic description of the index text to send or None to 
+      indicate the information is void of any known semantic
+    @type sem: integer
+    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
+      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @type layer: integer
+    @param por: por at which to place the marker
+    @type por: L{POR}
+
+    @todo: PP: rewrite to handle POR
     '''
     for mon in self.out_mons:
-      mon.showIndex(dev, marker)
+      mon.showIndex(dev, sem, layer, por)
     # for demonstration purposes, just print if no monitors
     #if not self.out_mons:
       #sys.stdout.write('{I:%s:%d}' % (dev, marker))
@@ -407,7 +408,7 @@ class DeviceManager(object):
     '''
     Registers the given device as an input device if the device implements the
     L{AEInput} base class and provides some subinterface not already provided
-    by another registered input device. For instance, if a device implementing
+    by another registered output device. For instance, if a device implementing
     L{AEInput.SystemInput} is already registered, another device implementing 
     just L{AEInput.SystemInput} will not be registered by this method.
     
@@ -437,16 +438,19 @@ class DeviceManager(object):
       self.def_input = dev
     return True
   
-  def _registerOutputDevice(self, dev):
+  def _registerOutputDevice(self, dev, num_styles):
     '''
     Registers the given device as an output device if the device implements the
     L{AEOutput} base class and provides some subinterface not already provided
     by another registered output device. For instance, if a device implementing
-    L{AEOutput.Speech} is already registered, another device implementing just
-    L{AEOutput.Speech} will not be registered by this method.
+    L{AEOutput.Audio} is already registered, another device implementing just
+    L{AEOutput.Audio} will not be registered by this method.
     
     @param dev: Device to attempt to register
     @type dev: L{AEOutput.AEOutput} class
+    @param num_styles: Number of distinct L{AEOutput.Style}s to requested from
+      the device
+    @type num_styles: integer 
     @return: True if the device was registered, False if it is not an 
       L{AEOutput} device, and None if it provides no new interfaces
     @rtype: boolean or None
@@ -454,6 +458,7 @@ class DeviceManager(object):
     if not implements(dev, AEOutput.AEOutput):
       # don't register something that isn't an output device
       return False
+    dev = dev.getProxy()  
     if self._isDuplicateDevice(dev, True):
       # don't register a device that provides all the same interfaces as a
       # previous device
@@ -473,9 +478,25 @@ class DeviceManager(object):
     # first output device registered is default
     if self.def_output is None:
       self.def_output = dev
+   
+    # create styles based on the abilities of this device
+    styles = dev.createDistinctStyles(num_styles)
+    # create semantics, layer to styles mapping for device
+    # todo: DBC: later, check to see if previously user-configured
+    for layer, i in enumerate(AEEvent.ALL_LAYERS):
+      # copy the style so we can change the channel
+      s = styles[0].copy()
+      s.Channel = i
+      # provide a default style for a semantic of None
+      self.styles[(None, layer, dev)] = s
+      for sem, grp in LSRConstants.SEMANTIC_STYLES.items():
+        s = styles[grp % num_styles].copy()
+        s.Channel = i
+        # map the style for (semantic, layer, device) using default mapping
+        self.styles[(sem, layer, dev)] = s
     return True
 
-  def registerDevice(self, dev):
+  def registerDevice(self, dev, num_styles=LSRConstants.DEFAULT_NUM_STYLES):
     '''    
     Registers the referenced device based on its one or more interfaces.
     When the interface is determined, the init() method is called on the device.
@@ -484,12 +505,15 @@ class DeviceManager(object):
     
     @param dev: The device to register
     @type dev: L{AEOutput} or L{AEInput} class
+    @param num_styles: Number of distinct L{AEOutput.Style}s to requested from
+      the device
+    @type num_styles: integer 
     @raise NotImplementedError: When the device implements none of the required
       interfaces
     @raise AEOutputError: When output device initialization fails
     @raise AEInputError: When input device initialization fails
     '''
-    is_out = self._registerOutputDevice(dev)
+    is_out = self._registerOutputDevice(dev, num_styles)
     is_in = self._registerInputDevice(dev)
 
     if is_out == False and is_in == False:
@@ -548,10 +572,10 @@ class DeviceManager(object):
     if dev in self.out_devs:
       self.def_output = dev
     elif implements(dev, AEOutput.AEOutput):
-      if self.register(dev):
+      if self.registerDevice(dev):
         self.def_output = dev
     else:
-      raise NotImplementedError(_('Not an AEOutput instance'))
+      raise NotImplementedError(_('Not an output device'))
 
   def getDefaultOutput(self):
     '''
@@ -594,7 +618,7 @@ class DeviceManager(object):
       if self.register(dev):
         self.def_input = dev
     else:
-      raise NotImplementedError(_('Not an AEInput instance'))
+      raise NotImplementedError(_('Not an input device'))
 
   def getDefaultInput(self):
     '''
@@ -638,16 +662,27 @@ class DeviceManager(object):
     @todo: PP: handle index callback markers
     '''
     pass
-
-if __name__ == '__main__':
-  # test this class
-  import DeviceManager
-  dm = DeviceManager.DeviceManager('not used')
-  dm.stopNow(None)
-  for ch in 'Hello World':
-    dm.sendChar(None, ch)
-  dm.talkNow(None)
-  dm.sendIndex(None,42)
-  dm.sendString(None,'Again')
   
+  def getStyle(self, dev, sem, layer):
+    '''
+    Gets the style for semantic/layer of a given device.
+    
+    @param dev: The device from which to get the style.
+    @type dev: L{AEOutput}
+    @param sem: The device semantic from which to get the style, or None to
+      get the device's default style.
+    @type sem: integer 
+    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
+      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @type layer: integer
+    @return: the L{AEOutput.Style} for the given semantic of a device
+    @rtype: L{AEOutput.Style}
+    @raise KeyError: When the semantic is invalid
+    '''
+    self.dirty_style = True
+    dev = dev or self.def_output
+    if sem is None:
+      return dev.getDefaultStyle() 
+    else:
+      return self.styles[(sem, layer, dev)]
 
