@@ -6,7 +6,7 @@ Defines the default user interface for LSR.
 @author: Larry Weiss
 @author: Brett Clippingdale
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005 IBM Corporation
+@copyright: Copyright (c) 2005, 2006 IBM Corporation
 @license: Common Public License 1.0
 
 All rights reserved. This program and the accompanying materials are made 
@@ -20,12 +20,20 @@ from i18n import _
 
 class DefaultPerk(Perk.Perk):
   '''
-  Creates and registers L{Task}s that will form the default user interface.
+  Defines a default user interface that makes LSR act like a typical screen 
+  reader. Event handlers are registered for focus, view, caret, and state change
+  events. Various commands for controling LSR are defined, but not mapped to
+  any particular input device.
+  
+  The following are stored in the L{perk_state} object for reference in all
+  classes in this L{Perk}.
   
   @ivar last_caret: Stores the previous caret L{POR}
   @type last_caret: L{POR}
   @ivar last_role: Stores the previous role to avoid duplicate announcements
   @type last_role: string
+  @ivar last_level: Stores the previous level to avoid duplicate announcements
+  @type last_level: integer
   '''
   def init(self):
     '''
@@ -33,10 +41,8 @@ class DefaultPerk(Perk.Perk):
     and L{HandleSelectorChange} objects to handle the associated events.
     Registers named L{Task}s that can be mapped to input gestures.
     '''
-    # create varibles referenced through self.this_perk
-    self.last_caret = POR(item_offset=0)
-    self.last_role = ''
-    self.last_menu = POR()
+    # create varibles referenced through self.perk
+    self.resetLasts()
     # register event handlers
     self.registerEventTask(HandleFocusChange(self))
     self.registerEventTask(HandleViewChange(self))
@@ -45,7 +51,8 @@ class DefaultPerk(Perk.Perk):
     self.registerEventTask(HandleStateChange(self))
     # register named tasks
     # these will be mapped to input gestures in other Perks
-    self.registerNamedTask(Stop(self), 'Stop speech now')
+    self.registerNamedTask(Stop(self), 'Stop now')
+    self.registerNamedTask(Mute(self), 'Mute/Unmute now')
     self.registerNamedTask(IncreaseRate(self), 'Increase rate')
     self.registerNamedTask(DecreaseRate(self), 'Decrease rate')
     self.registerNamedTask(PreviousItem(self), 'Previous item')
@@ -58,7 +65,8 @@ class DefaultPerk(Perk.Perk):
     self.registerNamedTask(CurrentChar(self), 'Current char')
     self.registerNamedTask(NextChar(self), 'Next char')
     self.registerNamedTask(WhereAmI(self), 'Where am I')
-    self.registerNamedTask(RoutePointer(self), 'Route pointer')
+    self.registerNamedTask(FocusToPointer(self), 'Focus to pointer')
+    self.registerNamedTask(PointerToFocus(self), 'Pointer to focus')
     self.registerNamedTask(ReadTop(self), 'Read top')
     self.registerNamedTask(ReadTextAttributes(self), 'Read text attributes')
     
@@ -77,14 +85,13 @@ class DefaultPerk(Perk.Perk):
     @rtype: boolean
     '''
     if task.hasAccRole('menu', por):
-      self.last_menu = por
+      self.perk_state.last_menu = por
       return False
     por = task.getParentAcc(por)
-    if task.hasAccRole('menu', por) and por != self.last_menu:
+    if task.hasAccRole('menu', por) and por != self.perk_state.last_menu:
       self.sayNewRole(task, por)
-      text = task.getItemText(por)
-      task.say(text)
-      self.last_menu = por
+      task.sayItem(por)
+      self.perk_state.last_menu = por
       return True
     return False
        
@@ -93,7 +100,6 @@ class DefaultPerk(Perk.Perk):
     Speaks the role of the given L{POR} or the L{task_por} if no L{POR} is given
     only if it is different than the last role spoken by this method.
     
-    @todo: PP: do not use hard coded voice numbers
     @param task: The task to use for calling L{Task.Tools} methods
     @type task: L{Task.Base}
     @param por: Point of regard to the accessible whose role should be said, 
@@ -103,46 +109,66 @@ class DefaultPerk(Perk.Perk):
     @rtype: boolean
     '''
     role = task.getAccRoleName(por)
-    if role != self.last_role:
-      task.setVoice(2)
-      task.say(role)
-      task.setVoice(1)
-      self.last_role = role
+    if role != self.perk_state.last_role:
+      task.sayRole(text=role)
+      self.perk_state.last_role = role
       return True
     return False
   
-  def sayState(self, task, por=None):
+  def sayNewLevel(self, task, por=None):
     '''
-    Speaks the state of the given L{POR} or the L{task_por} if no L{POR} is 
-    given only if it has some state worth reporting.
+    Speaks the level of the given L{POR} or the L{task_por} if no L{POR} is 
+    given only if it is different than the last level spoken by this method.
     
-    @todo: PP: do not use hard coded voice numbers
     @param task: The task to use for calling L{Task.Tools} methods
     @type task: L{Task.Base}
-    @param por: Point of regard to the accessible whose state should be said, 
+    @param por: Point of regard to the accessible whose role should be said, 
       defaults to L{task_por} if None
     @type por: L{POR}
-    @return: Was state announced?
+    @return: Was a new level announced?
     @rtype: boolean
     '''
-    states = task.getStateText(por)
-    if states:
-      task.setVoice(2)
-      task.say(states)
-      task.setVoice(1)
+    level = task.getAccLevel(por)
+    if level is not None and level != self.perk_state.last_level:
+      task.sayLevel(text=level, template=_('Level %d'))
+      self.perk_state.last_level = level
       return True
     return False
+  
+  def resetLasts(self):
+    '''
+    Resets variables tracking last announced role, level, and menu as well as
+    the last caret position.
+    '''
+    self.perk_state.last_caret = POR(item_offset=0)
+    self.perk_state.last_role = None
+    self.perk_state.last_level = None
+    self.perk_state.last_menu = POR()
 
 #
 # Tasks executed by keys
 #
 
+class Mute(Task.InputTask):
+  '''
+  Task to mute output indefinitely or unmute it if it has already been muted.
+  '''
+  def execute(self, **kwargs):
+    self.stopAll()
+    mute = self.getMute() 
+    if mute:
+      self.setMute(False)
+      self.sayInfo(text=_('Unmuted'))
+    else:
+      self.sayInfo(text=_('Muted'))
+      self.setMute(True)
+
 class Stop(Task.InputTask):
   '''
   Task to stop speech immediately, ignoring the value of the Stopping setting.
   '''
   def execute(self, **kwargs):
-    self.stopNow()
+    self.stopAll()
 
 class IncreaseRate(Task.InputTask):
   '''
@@ -183,9 +209,10 @@ class PreviousItem(Task.InputTask):
   def execute(self, **kwargs):
     self.stopNow()
     self.moveToPrevItem()
-    self.this_perk.sayNewRole(self)
+    self.perk.sayNewRole(self)
+    self.perk.sayNewLevel(self)
     self.sayItem()
-    self.this_perk.sayState(self)
+    self.sayState()
 
 class CurrentItem(Task.InputTask):
   '''
@@ -195,7 +222,7 @@ class CurrentItem(Task.InputTask):
     self.stopNow()
     self.moveToCurrItem()
     self.sayItem()
-    self.this_perk.sayState(self)
+    self.sayState()
 
 class NextItem(Task.InputTask):
   '''
@@ -204,9 +231,10 @@ class NextItem(Task.InputTask):
   def execute(self, **kwargs):
     self.stopNow()
     self.moveToNextItem()
-    self.this_perk.sayNewRole(self)
+    self.perk.sayNewRole(self)
+    self.perk.sayNewLevel(self)
     self.sayItem()
-    self.this_perk.sayState(self)
+    self.sayState()
     
 class PreviousWord(Task.InputTask):
   '''
@@ -216,10 +244,11 @@ class PreviousWord(Task.InputTask):
     self.stopNow()
     moved = self.moveToPrevWord()
     if moved:
-      self.this_perk.sayNewRole(self)
+      self.perk.sayNewRole(self)
+      self.perk.sayNewLevel(self)
     self.sayWord()
     if moved:
-      self.this_perk.sayState(self)
+      self.perk.sayState(self)
 
 class CurrentWord(Task.InputTask):
   '''
@@ -238,10 +267,11 @@ class NextWord(Task.InputTask):
     self.stopNow()
     moved = self.moveToNextWord()
     if moved:
-      self.this_perk.sayNewRole(self)
+      self.perk.sayNewRole(self)
+      self.perk.sayNewLevel(self)
     self.sayWord()
     if moved:
-      self.this_perk.sayState(self)
+      self.sayState()
     
 class PreviousChar(Task.InputTask):
   '''
@@ -251,10 +281,11 @@ class PreviousChar(Task.InputTask):
     self.stopNow()
     moved = self.moveToPrevChar()
     if moved:
-      self.this_perk.sayNewRole(self)
+      self.perk.sayNewRole(self)
+      self.perk.sayNewLevel(self)
     self.sayChar()
     if moved:
-      self.this_perk.sayState(self)
+      self.sayState()
 
 class CurrentChar(Task.InputTask):
   '''
@@ -273,59 +304,76 @@ class NextChar(Task.InputTask):
     self.stopNow()
     moved = self.moveToNextChar()
     if moved:
-      self.this_perk.sayNewRole(self)
+      self.perk.sayNewRole(self)
+      self.perk.sayNewLevel(self)
     self.sayChar()
     if moved:
-      self.this_perk.sayState(self)
+      self.sayState()
     
 class WhereAmI(Task.InputTask):
   '''
   Reports the location of the current L{task_por} by announcing all control and
   container names in child-to-parent order.
-  
-  @todo: PP: do not use hardcoded voice numbers
   '''
   def execute(self, **kwargs):
     # stop now and indicate the next stop is fine
     self.stopNow()
     pors = [self.task_por] + self.getAncestorAccs()
-    for curr in pors:
+    for curr in pors[:-1]:
       # announce role and text
-      self.setVoice(2)
-      self.say(self.getAccRoleName(curr))
-      self.setVoice(1)
+      self.sayRole(curr)
       self.sayItem(curr)
+    # announce window title in its appropriate semantic
+    curr = pors[-1]
+    self.sayRole(curr)
+    self.sayWindow(curr)
     # also announce the name of the application
-    self.setVoice(2)
-    self.say(_('application'))
-    self.setVoice(1)
-    self.say(self.getAppName())
+    self.sayRole(text=_('application'))
+    self.sayApp()
 
-class RoutePointer(Task.InputTask):
+class FocusToPointer(Task.InputTask):
   '''
   Attempts to switch focus to try to set focus to L{POR} of accessible that was
   last reviewed in passive mode.
   '''
   def execute(self, **kwargs):
+    self.stopNow()
     self.setFocus()
+    
+class PointerToFocus(Task.InputTask):
+  '''
+  Jumps the L{task_por} back to the L{POR} of the last focus event.
+  '''
+  def execute(self, **kwargs):
+    # set the Task POR to the focus POR
+    self.task_por = self.getFocus()
+    por = self.task_por
+    # stop all output and pretend a focus event happened to give context
+    self.mayStop()
+    # announce the label when present
+    self.sayLabel(por)
+    # announce parent menus when changing menus
+    self.perk.sayNewMenu(self, por)
+    # only announce role when it has changed
+    self.perk.sayNewRole(self, por)
+    # say the item text too
+    self.sayItem(por)
 
 class ReadTop(Task.InputTask):
   '''
   Read top of the view, typically the title of the foreground window.
   '''
   def execute(self, **kwargs):
-    por = self.task_por
-    frame_por = self.getRootAcc(por)
-    frame_name = self.getAccName(frame_por)
     self.stopNow()
-    self.say(frame_name)
+    self.sayWindow()
       
 class ReadTextAttributes(Task.InputTask):
   '''
   Read font attributes (size, style, color, etc).
   '''             
   def execute(self, **kwargs):
-    self.sayTextAttributes()
+    self.stopNow()
+    self.sayTextAttrs()
       
 # Tasks executed by events
 #
@@ -354,16 +402,13 @@ class HandleViewChange(Task.ViewTask):
     '''
     # trying stopping output
     self.stopNow()
-    # store the title to be announced on first focus
-    if title:
-      self.say(title)
-    else:
-      self.say(_('No title!'))
+    # say the title of the new view
+    if not self.sayWindow(text=title):
+      self.sayWindow(text=_('No title!'))
     # prevent the next focus event from stopping this announcement
     self.inhibitMayStop()
-    # reset last caret and role
-    self.this_perk.last_role = ''
-    self.this_perk.last_menu = POR()
+    # reset stateful vars
+    self.perk.resetLasts()
     return True
     
 class HandleFocusChange(Task.FocusTask):
@@ -379,40 +424,42 @@ class HandleFocusChange(Task.FocusTask):
     @type por: L{POR}
     '''
     self.mayStop()
-    # announce the label when present
-    label = self.getAccLabel()
-    if label:
-      self.say(label)
+    # announce the label when present if it is not the same as the name
+    label = self.getAccLabel(por)
+    if label and label != self.getItemText(por):
+      self.sayLabel(por)
     # announce parent menus when changing menus
-    self.this_perk.sayNewMenu(self, por)
+    self.perk.sayNewMenu(self, por)
     # only announce role when it has changed
-    self.this_perk.sayNewRole(self, por)
+    self.perk.sayNewRole(self, por)
     # prevent the next selector or caret from stopping this announcement
     self.inhibitMayStop()
-    # reset last_caret to ensure caret announcement
-    self.this_perk.last_caret = POR(item_offset=0)
+    # always reset last caret position
+    self.perk_state.last_caret = POR(item_offset=0)
     return True
 
 class HandleSelectorChange(Task.SelectorTask):
   '''  
   Task that handles a L{AEEvent.SelectorChange}.
   '''  
-  def execute(self, por, item_text, **kwargs):
+  def executeActive(self, por, text, **kwargs):
     '''
-    Announces the text of the selected item.
+    Announces the text of an actively selected item.
     
     @param por: Point of regard where the selection event occurred
     @type por: L{POR}
-    @param item_text: The text of the selected item
-    @type item_text: string
+    @param text: The text of the selected item
+    @type text: string
     '''
     self.mayStop()
     # only announce role when it has changed
-    self.this_perk.sayNewRole(self, por)
+    self.perk.sayNewRole(self, por)
+    # try to announce the level
+    self.perk.sayNewLevel(self, por)
     # announce the selected text
-    self.say(item_text)
+    self.sayItem(text=text)
     # see if there are any states worth reporting
-    self.this_perk.sayState(self, por)
+    self.sayState(por)
     return True
 
 class HandleCaretChange(Task.CaretTask):
@@ -420,6 +467,9 @@ class HandleCaretChange(Task.CaretTask):
   Task that handles a L{AEEvent.CaretChange}. The L{execute} method performs
   actions that should be done regardless of whether text was inserted or deleted
   or the caret moved. The other three methods handle these more specific cases.
+  
+  @todo: PP: think about semantic here for insert, move, and delete; is it echo?
+    is it char/word/item?
   '''  
   def execute(self, por, text, text_offset, added, **kwargs):
     '''
@@ -437,13 +487,13 @@ class HandleCaretChange(Task.CaretTask):
       (the default) when event is for caret movement only
     @type added: boolean
     '''
-    if not self.mayStop() and por.isSameAcc(self.this_perk.last_caret):
+    if not self.mayStop() and por.isSameAcc(self.perk_state.last_caret):
       # opt: bail out if we're trying to queue some other text event
       return True
     # let the base class call the appropriate method for inset, move, delete
     rv = Task.CaretTask.execute(self, por, text, text_offset, added, **kwargs)
     # store for next comparison
-    self.this_perk.last_caret = por
+    self.perk_state.last_caret = por
     return rv
     
   def executeInserted(self, por, text, text_offset, **kwargs):
@@ -459,7 +509,7 @@ class HandleCaretChange(Task.CaretTask):
     '''
     # inhibit the next stop
     self.inhibitMayStop()
-    self.say(text)
+    self.sayItem(text=text)
     return True
     
   def executeMoved(self, por, text, text_offset, **kwargs):
@@ -474,18 +524,18 @@ class HandleCaretChange(Task.CaretTask):
     @type text_offset: integer
     '''
     # say line when item_offset changes
-    if (not por.isSameItem(self.this_perk.last_caret) or
-        not por.isSameAcc(self.this_perk.last_caret)):
-      self.say(text)
+    if (not por.isSameItem(self.perk_state.last_caret) or
+        not por.isSameAcc(self.perk_state.last_caret)):
+      self.sayItem(text=text)
     # respond to caret movement on same line
     else: 
       # moved left, say text between positions
-      if por.isCharBefore(self.this_perk.last_caret):
-        self.say(text[por.char_offset : 
-                      self.this_perk.last_caret.char_offset])
+      if por.isCharBefore(self.perk_state.last_caret):
+        self.sayItem(text=text[por.char_offset : 
+                     self.perk_state.last_caret.char_offset])
       # moved right, say text between positions
-      elif por.isCharAfter(self.this_perk.last_caret):
-        self.say(text[self.this_perk.last_caret.char_offset : 
+      elif por.isCharAfter(self.perk_state.last_caret):
+        self.sayItem(text=text[self.perk_state.last_caret.char_offset : 
                       por.char_offset])
     return True
     
@@ -501,15 +551,15 @@ class HandleCaretChange(Task.CaretTask):
     @param text_offset: The offset of the delete text
     @type text_offset: integer
     '''   
-    if (self.this_perk.last_caret.item_offset + 
-        self.this_perk.last_caret.char_offset) > text_offset:
+    if (self.perk_state.last_caret.item_offset + 
+        self.perk_state.last_caret.char_offset) > text_offset:
       # inhibit the next stop
       self.inhibitMayStop()
       # say backspace if we're backing up
-      self.say(_('backspace %s') % text)
+      self.sayItem(text=text, template=_('backspace %s'))
     else:
       # say delete if we're deleting the current
-      self.say(_('delete %s') % text)
+      self.sayItem(text=text, template=_('delete %s'))
     return True
 
   def update(self, por, text, text_offset, added, **kwargs):
@@ -528,7 +578,7 @@ class HandleCaretChange(Task.CaretTask):
     @type added: boolean
     '''
     # store for next comparison
-    self.this_perk.last_caret = por
+    self.perk_state.last_caret = por
   
 class HandleStateChange(Task.StateTask):
   '''  
@@ -550,5 +600,5 @@ class HandleStateChange(Task.StateTask):
     if text:
       # try a may stop
       self.mayStop()
-      self.say(text)
-    return True
\ No newline at end of file
+      self.sayState(text=text)
+    return True
