@@ -3,7 +3,7 @@ Defines a L{Task} to execute when a L{AEEvent.InputGesture} occurs.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005 IBM Corporation
+@copyright: Copyright (c) 2005, 2006 IBM Corporation
 @license: Common Public License 1.0
 
 All rights reserved. This program and the accompanying materials are made 
@@ -17,12 +17,12 @@ class InputTask(Base.Task):
   '''
   Executes when an input gesture occurs.
 
-  This class registers its name and whether it should be monitored by default in
-  an L{AEMonitor.EventMonitor} using the L{Constants.registerTaskType} function
-  when this module is first imported. The L{AEMonitor.EventMonitor} can use this
-  information to build its menus.
+  This class registers its name and whether it should be monitored by default
+  in an L{AEMonitor.EventMonitor} using the L{Constants.registerTaskType}
+  function when this module is first imported. The L{AEMonitor.EventMonitor}
+  can use this information to build its menus.  
   '''
-  Constants.registerTaskType('InputTask', True)
+  Constants.registerTaskType('InputTask', False)
   
   def execute(self, **kwargs):
     '''
