@@ -19,12 +19,6 @@ particular L{Tier} starts.
 @type INSTALL_PATH: string
 @var PROFILE_PATH: Path where serialized L{Profile}s live
 @type PROFILE_PATH: string
-@var STARTUP: Indicates UIEs to be loaded on LSR startup
-@type STARTUP: string
-@var ONE_TIER: Indicates UIEs to be loaded when a particular L{Tier} is created
-@type ONE_TIER: string
-@var ANY_TIER: Indicates UIEs to be loaded when any L{Tier} is created
-@type ANY_TIER: string
 @var PERK: Kind of UIE responsible for registering L{Task}s that handle 
   L{AEEvent}s and L{AEInput.Gesture}s
 @type PERK: string
@@ -41,10 +35,12 @@ particular L{Tier} starts.
 @type MONITOR: string
 @var ALL_KINDS: List of all known kinds of UIEs
 @type ALL_KINDS: list
+@var STARTUP_KINDS: Subset of L{ALL_KINDS} that should be loaded at startup
+@type STARTUP_KINDS: list
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005 IBM Corporation
+@copyright: Copyright (c) 2005, 2006 IBM Corporation
 @license: Common Public License 1.0
 
 All rights reserved. This program and the accompanying materials are made 
@@ -53,7 +49,6 @@ this distribution, and is available at
 U{http://www.opensource.org/licenses/cpl1.0.php}
 '''
 import os, glob, imp, cPickle, sys, logging
-import AEMonitor, AEChooser, AEOutput, AEInput, Perk
 from i18n import _
 from LSRConstants import HOME_USER
 
@@ -70,11 +65,6 @@ INSTALL_PATH = os.path.join(ROOT_PATH, 'installed')
 PROFILE_PATH = os.path.join(ROOT_PATH, 'profiles')
 VERSION_PATH = os.path.join(ROOT_PATH, 'version')
 
-# constants representing when associated UIEs are loaded
-STARTUP = 'startup'
-ONE_TIER = 'onetier'
-ANY_TIER = 'anytier'
-
 # constants representing kinds of UIEs and the folders where they are stored 
 # under the INSTALL_PATH
 PERK = 'perk'
@@ -84,15 +74,14 @@ MONITOR = 'monitor'
 
 # list of all supports types of UIEs
 ALL_KINDS = [DEVICE, CHOOSER, MONITOR, PERK]
-ALL_TIMES = [STARTUP, ONE_TIER, ANY_TIER]
+STARTUP_KINDS = [DEVICE, CHOOSER, MONITOR]
 
 class Profile(object):
-  '''
+  '''  
   Associates the names of installed UIEs with times when they should be
   automatically loaded by LSR under a particular user profile. Supported times
-  for automatically loading UIEs include when LSR starts (L{STARTUP}), when any
-  new L{Tier} is created (L{ANY_TIER}), or when a L{Tier} with a particular name
-  is created (L{ONE_TIER}).
+  for automatically loading UIEs include when LSR starts, when any new L{Tier}
+  is created , or when a L{Tier} with a particular process name is created.
   
   @ivar name: Name of this L{Profile}
   @type name: string
@@ -414,15 +403,14 @@ class UIRegistrar(object):
   methods. See the documentation in that module for details.
   
   Methods to load UIEs are of interest to LSR developers. Given a UIE kind and
-  name, the L{loadOne} method will import the Python module containing the named
-  UIE and return a reference to the class in that module of the same name (if
-  the UIE module is properly installed). Given a UIE kind and name along with
-  the name of a L{Profile} and a indicator of when the request is taking place
-  (L{STARTUP}, L{ONE_TIER}, L{ANY_TIER}), the L{loadAssociated} method will
-  import all Python modules containing the UIEs to be loaded at the indicated
-  time and return a list of UIE objects sorted in the desired load order. 
-  Objects, not classes, are returned by this method implying any initialization
-  requiring data from an external source must be done outside the constructor.
+  name, the L{loadOne} method will import the Python module containing the
+  named UIE and return a reference to the class in that module of the same
+  name (if the UIE module is properly installed). Given a UIE kind and name
+  along with the name of a L{Profile}, the L{loadAssociated} method will
+  import all Python modules containing the UIEs to be loaded and return a list
+  of UIE objects sorted in the desired load order. Objects, not classes, are
+  returned by this method implying any initialization requiring data from an
+  external source must be done outside the constructor.
   '''
   def _ensurePathsExist(self):
     '''
@@ -477,29 +465,6 @@ class UIRegistrar(object):
     '''
     if self._isInstalled(kind, name):
       raise ValueError(_('A %s with name %s is already installed')%(kind, name))
-    
-  def _ensureAssociationValid(self, kind, when):
-    '''
-    Ensures the given kind of UIE can be loaded at the time given by when. At
-    present, the following pairings are valid:
-    
-    Chooser: None
-    Device: STARTUP
-    Monitor: STARTUP
-    Perk: ANY_TIER, ONE_TIER
-    
-    @param kind: Kind of UIE, one of L{ALL_KINDS}
-    @type kind: string
-    @param when: Identifier of when the UIE will be loaded automatically: on
-       LSR startup (L{STARTUP}), on creation of any L{Tier} (L{ANY_TIER}), or
-       on creation of a particular L{Tier} (L{ONE_TIER})
-    @type when: string
-    @raise ValueError: When the pairing is invalid
-    '''
-    if kind == PERK and when in (ANY_TIER, ONE_TIER):
-      return
-    elif kind == CHOOSER or when != STARTUP:
-      raise ValueError(_('A %s cannot be loaded on %s') % (kind, when))
   
   def _ensureUIEValid(self, kind, name, real_path):
     '''    
@@ -695,13 +660,10 @@ class UIRegistrar(object):
       profile.removeFromAll(kind, name)
       profile.save()
   
-  def associate(self, kind, name, profile_name, when, tier_name=None, 
-                index=None):
+  def associate(self, kind, name, profile_name, tier_name=None, index=None):
     '''
     Associates an installed UIE of the given kind and name with a profile so 
-    that is loaded automatically by LSR at the time specified. Ensures the
-    desired UIE is installed and that the when parameter is valid for the given
-    UIE kind via the L{_ensureAssociationValid}.
+    that is loaded automatically by LSR at the time specified.
     
     @param kind: Kind of UIE, one of L{ALL_KINDS}
     @type kind: string
@@ -710,12 +672,8 @@ class UIRegistrar(object):
     @param profile_name: Name of the profile in which UIE of the given name and
       kind will be loaded automatically
     @type profile_name: string
-    @param when: Identifier of when the UIE will be loaded automatically: on
-     LSR startup (L{STARTUP}), on creation of any L{Tier} (L{ANY_TIER}), or on
-     creation of a particular L{Tier} (L{ONE_TIER})
-    @type when: string
     @param tier_name: Name of the L{Tier} which will cause the loading of the
-      given UIE. Only used when the previous parameter is set to L{ONE_TIER}
+      given UIE
     @type tier_name: string
     @param index: Load order of the UIE when multiple UIEs are to be loaded at
       the given time. Defaults to None meaning after all other associated UIEs
@@ -728,21 +686,19 @@ class UIRegistrar(object):
     '''
     # make sure the item is installed
     self._ensureInstalled(kind, name)
-    # make sure the UIE can be loaded at the time given by when
-    self._ensureAssociationValid(kind, when)
     # load the profile, will raise ValueError if not found
-    profile = Profile.load(profile_name)
+    profile = Profile.load(profile_name)   
     # decide when to load the give UIE
-    if when == STARTUP:
+    if kind in STARTUP_KINDS:
       profile.addToStartup(kind, name, index)
-    elif when == ONE_TIER:
+    elif tier_name is not None:
       profile.addToTier(kind, name, tier_name, index)
-    elif when == ANY_TIER:
+    else:
       profile.addToAnyTier(kind, name, index)
     # write the profile back to disk
     profile.save()
   
-  def disassociate(self, kind, name, profile_name, when, tier_name=None):
+  def disassociate(self, kind, name, profile_name, tier_name=None):
     '''
     Disassociates an installed UIE of the given kind and name with a profile so 
     that it is no longer loaded automatically by LSR at the time specified.
@@ -754,13 +710,8 @@ class UIRegistrar(object):
     @param profile_name: Name of the profile in which UIE of the given name and
         kind will no longer be loaded automatically
     @type profile_name: string
-    @param when: Identifier of when the UIE will be disassociated: on LSR 
-       startup (L{STARTUP}), on creation of any L{Tier} (L{ANY_TIER}), or on
-       creation of a particular L{Tier} (L{ONE_TIER})
-    @type when: string
     @param tier_name: Name of the L{Tier} which will no longer cause the loading
-        of the given UIE. Only used when the previous parameter is set to 
-        L{ONE_TIER}
+        of the given UIE
     @type tier_name: string
     @raise ValueError: When a UIE with the given kind and name is not installed
     @raise IOError: When the registrar cannot write to the profiles folder
@@ -768,11 +719,11 @@ class UIRegistrar(object):
     # load the profile, will raise ValueError if not found
     profile = Profile.load(profile_name)
     # decide when not to load the give UIE
-    if when == STARTUP:
+    if kind in STARTUP_KINDS:
       profile.removeFromStartup(kind, name)
-    elif when == ONE_TIER:
+    elif tier_name is not None:
       profile.removeFromTier(kind, name, tier_name)
-    elif when == ANY_TIER:
+    else:
       profile.removeFromAnyTier(kind, name)
     # write the profile back to disk
     profile.save()
@@ -805,7 +756,8 @@ class UIRegistrar(object):
     # convert the set to a list
     return list(s)
   
-  def listAssociated(self, kind, profile_name, when, tier_name=None):
+  def listAssociated(self, kind, profile_name, tier_name=None, 
+                     all_tier_names=False):
     '''
     Gets a list of the names of all UIEs of the given kind associated with the
     given profile to be loaded at the specified time.
@@ -814,13 +766,11 @@ class UIRegistrar(object):
     @type kind: string
     @param profile_name: Name of the profile
     @type profile_name: string
-    @param when: Identifier of the time when the listed UIEs are loaded: on LSR 
-       startup (L{STARTUP}), on creation of any L{Tier} (L{ANY_TIER}), or on
-       creation of a particular L{Tier} (L{ONE_TIER})
-    @type when: string
     @param tier_name: Name of the L{Tier} which causes the loading of the listed
-        UIEs. Only used when the previous parameter is set to L{ONE_TIER}
+        UIEs
     @type tier_name: string
+    @param all_tier_names: List all UIEs associated with named L{Tier}s?
+    @type all_tier_names: boolean
     @return: Names of all UIEs of the given kind associated with the given 
         profile to be loaded at the given time
     @rtype: list of string
@@ -828,11 +778,14 @@ class UIRegistrar(object):
     # load the profile, will raise ValueError if not found
     profile = Profile.load(profile_name)
     # decide when not to load the give UIE
-    if when == STARTUP:
+    if kind in STARTUP_KINDS:
       return profile.listStartup(kind)
-    elif when == ONE_TIER:
+    elif tier_name is not None:
       return profile.listTier(kind, tier_name)
-    elif when == ANY_TIER:
+    elif all_tier_names:
+      # list all named tiers
+      return profile.listTier(kind, None)
+    else:
       return profile.listAnyTier(kind)
   
   def loadOne(self, kind, name):
@@ -858,7 +811,7 @@ class UIRegistrar(object):
       log.info('Error %s loading UIE: %s', str(e), name)
       return None
   
-  def loadAssociated(self, kind, profile_name, when, tier_name=None):
+  def loadAssociated(self, kind, profile_name, tier_name=None):
     '''
     Gets all UIE classes in the given profile to be loaded at the given time.
     
@@ -866,19 +819,15 @@ class UIRegistrar(object):
     @type kind: string
     @param profile_name: Name of the profile
     @type profile_name: string
-    @param when: Identifier of the time when the listed UIEs are loaded: on LSR 
-       startup (L{STARTUP}), on creation of any L{Tier} (L{ANY_TIER}), or on
-       creation of a particular L{Tier} (L{ONE_TIER})
-    @type when: string
     @param tier_name: Name of the L{Tier} which causes the loading of the listed
-        UIEs. Only used when the previous parameter is set to L{ONE_TIER}
+        UIEs
     @type tier_name: string
     @return: All UIE classes to be loaded at this time sorted proper load order
     @rtype: list of class
     @raise ValueError: When any associated UIE with the given kind and name is 
         not installed
     '''
-    names = self.listAssociated(kind, profile_name, when, tier_name)
+    names = self.listAssociated(kind, profile_name, tier_name)
     uies = []
     for name in names:
       obj = self.loadOne(kind, name)
